/*
Copyright (c) 2003-2019, CKSource - Frederico Knabben. All rights reserved.
For licensing, see LICENSE.md or https://ckeditor.com/legal/ckeditor-oss-license
*/
CKEDITOR.plugins.setLang( 'stylescombo', 'ro', {
	label: 'Stil',
	panelTitle: 'Formatare stilurilor',
	panelTitle1: 'Bloc stiluri',
	panelTitle2: 'Stiluri înșiruite',
	panelTitle3: 'Stiluri obiect'
} );
