/*
Copyright (c) 2003-2019, CKSource - Frederico Knabben. All rights reserved.
For licensing, see LICENSE.md or https://ckeditor.com/legal/ckeditor-oss-license
*/
CKEDITOR.plugins.setLang( 'docprops', 'es', {
	bgColor: 'Color de fondo',
	bgFixed: 'Fondo fijo (no se desplaza)',
	bgImage: 'Imagen de fondo',
	charset: 'Codificación de caracteres',
	charsetASCII: 'ASCII',
	charsetCE: 'Centro Europeo',
	charsetCR: 'Ruso',
	charsetCT: 'Chino Tradicional (Big5)',
	charsetGR: 'Griego',
	charsetJP: 'Japonés',
	charsetKR: 'Koreano',
	charsetOther: 'Otra codificación de caracteres',
	charsetTR: 'Turco',
	charsetUN: 'Unicode (UTF-8)',
	charsetWE: 'Europeo occidental',
	chooseColor: 'Elegir',
	design: 'Diseño',
	docTitle: 'Título de página',
	docType: 'Tipo de documento',
	docTypeOther: 'Otro tipo de documento',
	label: 'Propiedades del documento',
	margin: 'Márgenes',
	marginBottom: 'Inferior',
	marginLeft: 'Izquierdo',
	marginRight: 'Derecho',
	marginTop: 'Superior',
	meta: 'Meta Tags',
	metaAuthor: 'Autor',
	metaCopyright: 'Copyright',
	metaDescription: 'Descripción del documento',
	metaKeywords: 'Palabras claves del documento separadas por coma (meta keywords)',
	other: 'Otro...',
	previewHtml: '<p>Este es un <strong>texto de ejemplo</strong>. Usted está usando <a href="javascript:void(0)">CKEditor</a>.</p>',
	title: 'Propiedades del documento',
	txtColor: 'Color del texto',
	xhtmlDec: 'Incluir declaración XHTML'
} );
