/*
Copyright (c) 2003-2019, CKSource - Frederico Knabben. All rights reserved.
For licensing, see LICENSE.md or https://ckeditor.com/legal/ckeditor-oss-license
*/
CKEDITOR.plugins.setLang( 'fakeobjects', 'bn', {
	anchor: 'Anchor', // MISSING
	flash: 'Flash Animation', // MISSING
	hiddenfield: 'Hidden Field', // MISSING
	iframe: 'IFrame', // MISSING
	unknown: 'Unknown Object' // MISSING
} );
