/*
Copyright (c) 2003-2019, CKSource - Frederico Knabben. All rights reserved.
For licensing, see LICENSE.md or https://ckeditor.com/legal/ckeditor-oss-license
*/
CKEDITOR.plugins.setLang( 'fakeobjects', 'eu', {
	anchor: 'Aingura',
	flash: 'Flash animazioa',
	hiddenfield: 'Ezkutuko eremua',
	iframe: 'IFrame-a',
	unknown: 'Objektu ezezaguna'
} );
