/*
Copyright (c) 2003-2019, CKSource - Frederico Knabben. All rights reserved.
For licensing, see LICENSE.md or https://ckeditor.com/legal/ckeditor-oss-license
*/
CKEDITOR.plugins.setLang( 'fakeobjects', 'ru', {
	anchor: 'Якорь',
	flash: 'Flash анимация',
	hiddenfield: 'Скрытое поле',
	iframe: 'iFrame',
	unknown: 'Неизвестный объект'
} );
